using System;
using System.Collections.Generic;
using System.Text;
using Sedodream.Sudoku.Board;
using Sedodream.Sudoku.Evaluators;

namespace Sedodream.Sudoku.Players
{
	public class SimplePlayer : IPlayer
	{
		#region Fields
		private int numMovesPlayed = 0;
		private int maxNumMovesPlayed;
		private MoveScoreComparer moveComparer = new MoveScoreComparer();
		private IEvaluator evaluator;
		//private int numMovesPlayed;
		
		#endregion

		#region Constructors
		public SimplePlayer()
		{
			numMovesPlayed = 0;
			maxNumMovesPlayed = int.MaxValue;
			moveComparer = new MoveScoreComparer();
			evaluator = new SimpleEvaluator();
		}
		#endregion

		private IEvaluator Evaluator
		{
			get { return this.evaluator; }
			set { this.evaluator = value; }
		}
		private MoveScoreComparer MoveComparer
		{
			get { return this.moveComparer; }
			set { this.moveComparer = value; }
		}

		#region IPlayer Members

		public IBoard StartPlay(IBoard board)
		{
			IBoard solvedBoard = null;
			bool solved = AdvanceBoard(board, out solvedBoard);
			if (solved)
			{
				int debug = 33;
			}
			return solvedBoard;
		}

		public int MaxNumMoves
		{
			get { return this.maxNumMovesPlayed; }
			set { this.maxNumMovesPlayed = value; }
		}
		public int NumMovesPlayed
		{
			get { return this.numMovesPlayed; }
		}
		#endregion
		private bool AdvanceBoard(IBoard board, out IBoard solvedBoard)
		{
			solvedBoard = null;
			//return false;
		
			if(board.State==BoardState.Canceled)
			{
				return false;
			}
			board.UpdateBoardState();
			if (board.State == BoardState.Solved)
			{
				return true;
			}
			if (board.State == BoardState.Failed)
			{
				return false;
			}
			List<IMove> moveList = board.FindAllMoves();
			if (moveList == null || moveList.Count <= 0)
			{
				return false;
			}

			foreach (IMove test in moveList)
			{
				if (test == null)
				{
					string d = "where can we go with this";
				}
				if (!board.IsMoveValid(test))
				{
					string why = "WHY";
				}
			}

			int numForcedMoves = 0;
			//cycle through and play all forced moves
			IList<IMove> forcedMovesPlayed = new List<IMove>();
			bool hasForcedMoves = true;
			while (hasForcedMoves)
			{
				foreach (IMove move in moveList)
				{
					if (move.ForcedMove)
					{
						numForcedMoves++;
						BoardState bSstate = PlayMove(board, move);
						forcedMovesPlayed.Add(move);
						if (bSstate == BoardState.Solved)
						{
							solvedBoard = board;
							return true;
						}
						if (bSstate == BoardState.Failed)
						{
							//							board.UpdateBoardState();
							string stateMessage = string.Format("Invalid state due to playing forced move");
							return false;
							//throw new ApplicationException(stateMessage);
						}
						if (bSstate == BoardState.Canceled)
						{
							return false;
						}
					}
					else
					{
						//moves should be sorted, forced moves first
						break;
					}
				}
				foreach (IMove move in forcedMovesPlayed)
				{
					if (moveList.Contains(move))
						moveList.Remove(move);
				}
				forcedMovesPlayed.Clear();
				moveList = board.FindAllMoves();
				if (moveList != null && moveList.Count > 0 && moveList[0] != null && moveList[0].ForcedMove)
					hasForcedMoves = true;
				else
					hasForcedMoves = false;
			}
			
			board.UpdateBoardState();
			BoardState state = board.State;
			if (state == BoardState.Solved)
			{
				solvedBoard = board;
				return true;
			}
			if (state == BoardState.Failed)
			{
				string stateMessage = string.Format("Invalid state due to playing forced move");
				throw new ApplicationException(stateMessage);
			}
			moveList = board.FindAllMoves();
			//TODO: Remove failing moves
			DateTime start = DateTime.Now;

			if (moveList != null && moveList.Count > 0)
			{
				//now we have to score each move
				foreach (IMove move in moveList)
				{
					move.MoveScore = Evaluator.Evaluate(this, move,moveList);
					int debug = 3;
				}
				//now we need to sort the moves
				moveList.Sort(moveComparer);
			}

			foreach (IMove test in moveList)
			{
				if (test == null)
				{
					string d = "where can we go with this";
				}
				if (!board.IsMoveValid(test))
				{
					string why = "WHY";
				}
			}

			TimeSpan timeSpent = DateTime.Now.Subtract(start);
			foreach (IMove move in moveList)
			{
				//IBoard newBoard = new SudokuBoard((IBoardData)board.BoardData.Clone());

				//first see if the move is still valid
				if (!board.IsMoveValid(move))
					continue;

				IBoard newBoard = new SudokuBoard(board.BoardData);

				//TODO: REMOVE------------------------------------------
				//bool isValid = board.IsMoveValid(move);
				//------------------------------------------------------

				BoardState newState = PlayMove(newBoard, move);
				
				if (newState == BoardState.Solved)
				{
					return true;
				}
				if (newState == BoardState.Canceled)
				{
					return false;
				}
				if (AdvanceBoard(newBoard, out solvedBoard) == true)
				{
					if (solvedBoard == null)
						solvedBoard = newBoard;
					return true;
				}
				this.UndoMove(newBoard, move);
				if (newBoard.State == BoardState.Failed)
				{
					return false;
				}
				if (newBoard.State == BoardState.Solved)
				{
					//SHOULD NEVER GET HERE
					if (solvedBoard == null)
						solvedBoard = newBoard;
					return true;
				}
			}
			if (board.State == BoardState.Solved)
			{
				int debug = 3;
			}
			return (board.State==BoardState.Solved);
		}
		public void UndoMove(IBoard board, IMove move)
		{
			if (board == null)
				throw new ArgumentNullException("board");
			if (move == null)
				throw new ArgumentNullException("move");
			board.SetValueAt(move.Location, null);
			//board.Cells[move.Location.Row,move.Location.Col].Cell
			//TODO: Validate that this move has been played on the board
			
		}
		public BoardState PlayMove(IBoard board, IMove move)
		{
			if (board.State == BoardState.Canceled)
			{
				return BoardState.Canceled;
			}
			if (numMovesPlayed > MaxNumMoves)
			{
				board.State = BoardState.Canceled;
				return BoardState.Canceled;
			}
			if (board == null)
				throw new ArgumentNullException("board");
			if (move == null)
				throw new ArgumentNullException("move");

			if (move.Location.Row == 3 && move.Location.Col == 0 && move.Value == 3)
			{
				string messgae = "problem point";
			}
			if (!board.IsValueValid(move.Location, move.Value))
			{
				string test3 = "Invalid value";
			}

			board.SetValueAt(move.Location, move.Value);
			board.UpdateBoardState();
			if (board.State == BoardState.Failed)
			{
				int debug = 3;
			}
			numMovesPlayed++;
			if (numMovesPlayed > MaxNumMoves)
			{
				board.State = BoardState.Canceled;
				return board.State;
			}
			return board.State;
		}
	}
}
